//+------------------------------------------------------------------+
//| EA #2                                                            |
//| =====                                                            |
//| Created: 18.11.2008                                              |
//| Updated: 20.11.2008                                              |
//| Author : AdamP                                                   |
//+------------------------------------------------------------------+

#include       <common.mqh>

#define        MAXORDERS            5


/**
 * Technical Indicator Variables
 */
extern int     FastMA            =  8;
extern int     MediumMA          =  21;
extern int     SlowMA            =  55;
extern int     TrailStop         =  200;
extern int     InitialSL         =  200;


/**
 * Check for bullish market conditions
 */
bool isBullishMarket()
{
   double fast = iMA(Symbol(),PERIOD_W1,FastMA,0,MODE_EMA,PRICE_CLOSE,0);
   double medium = iMA(Symbol(),PERIOD_W1,MediumMA,0,MODE_EMA,PRICE_CLOSE,0);
   double slow = iMA(Symbol(),PERIOD_W1,SlowMA,0,MODE_EMA,PRICE_CLOSE,0);
   
   // Bullish: fast > medium > slow in WEEKLY chart
   if ((fast > medium) && (medium > slow))
   {
      return (true);
   }
   else
   {
      return (false);
   }
}


/**
 * Check for bearish market conditions
 */
bool isBearishMarket()
{
   double fast = iMA(Symbol(),PERIOD_W1,FastMA,0,MODE_EMA,PRICE_CLOSE,0);
   double medium = iMA(Symbol(),PERIOD_W1,MediumMA,0,MODE_EMA,PRICE_CLOSE,0);
   double slow = iMA(Symbol(),PERIOD_W1,SlowMA,0,MODE_EMA,PRICE_CLOSE,0);
   
   // Bullish: fast < medium < slow in WEEKLY chart
   if ((fast < medium) && (medium < slow))
   {
      return (true);
   }
   else
   {
      return (false);
   }
}


/**
 * Look out for a buy signal
 */
bool buySignalOccured()
{
   double currentFast = iMA(Symbol(),PERIOD_D1,FastMA,0,MODE_EMA,PRICE_CLOSE,0);
   double previousMedium = iMA(Symbol(),PERIOD_D1,MediumMA,0,MODE_EMA,PRICE_CLOSE,1);
   double currentMedium = iMA(Symbol(),PERIOD_D1,MediumMA,0,MODE_EMA,PRICE_CLOSE,0);
   double previousSlow = iMA(Symbol(),PERIOD_D1,SlowMA,0,MODE_EMA,PRICE_CLOSE,1);
   double currentSlow = iMA(Symbol(),PERIOD_D1,SlowMA,0,MODE_EMA,PRICE_CLOSE,0);
   
   if (currentFast > currentMedium)
   {
      if ((previousMedium < previousSlow) && (currentMedium > currentSlow))
         return (true);
   }
   return (false);
}


/**
 * Look out for a sell signal
 */
bool sellSignalOccured()
{
   double currentFast = iMA(Symbol(),PERIOD_D1,FastMA,0,MODE_EMA,PRICE_CLOSE,0);
   double previousMedium = iMA(Symbol(),PERIOD_D1,MediumMA,0,MODE_EMA,PRICE_CLOSE,1);
   double currentMedium = iMA(Symbol(),PERIOD_D1,MediumMA,0,MODE_EMA,PRICE_CLOSE,0);
   double previousSlow = iMA(Symbol(),PERIOD_D1,SlowMA,0,MODE_EMA,PRICE_CLOSE,1);
   double currentSlow = iMA(Symbol(),PERIOD_D1,SlowMA,0,MODE_EMA,PRICE_CLOSE,0);
   
   if (currentFast < currentMedium)
   {
      if ((previousMedium > previousSlow) && (currentMedium < currentSlow))
         return (true);
   }
   return (false);   
}


/**
 * Check conditions for sell or buy a position
 */
void checkForOpen()
{
   double lot;

   // Only trade on beginning of a new bar
   if (Volume[0] > 1) return;
   
   // Check for buy condition on selected pair
   if (isBullishMarket())
   {
      if (buySignalOccured())
      {
         lot = lotsOptimized(InitialSL*Point);
         newMarketOrder(Symbol(),"BUY",lot,Ask-InitialSL*Point,0);
      }
   }

   // Check for sell condition on selected pair
   if (isBearishMarket())
   {
      if (sellSignalOccured())
      {
         lot = lotsOptimized(InitialSL*Point);
         newMarketOrder(Symbol(),"SELL",lot,Bid+InitialSL*Point,0);
      }
   }
}


/**
 * Update trailing stops
 */
void updateTrailingStops()
{   
      double stoploss;
      
      // Iterate through each order and pick the ones that belong to our EA
      for(int j=0;j<OrdersTotal();j++)
      {
         if(OrderSelect(j,SELECT_BY_POS,MODE_TRADES)==false)
            break;
         
         if(OrderMagicNumber()!=MAGICNUMBER || OrderSymbol()!=Symbol())
            continue;
      
         if(OrderType()==OP_BUY)
         {
            // Is order already in profit zone?
            if (Ask-TrailStop*Point > OrderOpenPrice())
               // Is new loss above current orders stoploss?
               if (Ask-TrailStop*Point > OrderStopLoss())
                  OrderModify(OrderTicket(),OrderOpenPrice(),Ask-TrailStop*Point,0,0,Blue);
            continue;
         }
         if(OrderType()==OP_SELL)
         {
            // Is order already in profit zone?
            if (Bid+TrailStop*Point < OrderOpenPrice())
               // Is new loss below current orders stoploss?
               if (Bid+TrailStop*Point < OrderStopLoss())   
                  OrderModify(OrderTicket(),OrderOpenPrice(),Bid+TrailStop*Point,0,0,Blue);
            continue;
         }
      }
}



/**
 * Starting point of expert
 */
void start()
{      
   if (MediumMA <= FastMA)
      return;
   if (SlowMA <= MediumMA)
      return;
 
   // Update our order and money managers  
   updateMoneyManager();
   updateOrderManager();
     
   // Update trailing stops - we don't use a sell signal to close a position,
   // this could be implemented later. But trailing stops should give good results either
   updateTrailingStops();
   
   // Check for open a position
   if (countOpenOrders() < MAXORDERS)
      checkForOpen();
}