//+------------------------------------------------------------------+
//|                                                       common.mq4 |
//|                                   Copyright  2008, Adam Parusel |
//|                                                http://esolvix.de |
//+------------------------------------------------------------------+
#property copyright "Copyright  2008, Adam Parusel"
#property link      "http://esolvix.de"

#define MAGICNUMBER  29031985

// Maximum of allowed slipage for order execution
#define SLIPAGE      5

// Save all open orders in this array
int      Ticket[];

/**
 * Count open positions for a given currency and save our orders in the ticket-array.
 * We cannot simply return ArraySize(Ticket), because some orders could already
 * have been stopped out.
 */
int countOpenOrders()
{
   return (updateOrderManager());
}


/**
 * Opens a new position and returns the error code
 *
 * @param   pair        currency pair, use "EURUSD" or Symbol()
 * @param   command     "SELL" or "SELL" - don't use OP_BUY or OP_SELL
 * @param   lot         size of order in lot
 * @param   stoploss    stoploss
 * @param   takeprofit  takeprofit
 *
 */
int newMarketOrder(string pair, string command, double lot, double stoploss, double takeprofit)
{
   // Buy command send
   if (command == "BUY" || command == "buy")
   {
      return (OrderSend(pair,OP_BUY,lot,Ask,SLIPAGE,stoploss,takeprofit,"",MAGICNUMBER,0,Green));
   }
   // Sell command send
   else if (command == "SELL" || command == "sell")
   {
      return (OrderSend(pair,OP_SELL,lot,Bid,SLIPAGE,stoploss,takeprofit,"",MAGICNUMBER,0,Red));
   }
   else return (-2);
}


/**
 * Updates all informations about the currently opened orders.
 * Especially looks through all orders if they are still active or
 * maybe have been stopped out by stop losses and saves their ticket
 * number in Ticket array.
 **/
int updateOrderManager()
{
   int count=0;
   
   // Resize ticket array to zero
   ArrayResize(Ticket,0);

   // Go through all orders
   for(int i=0;i<OrdersTotal();i++)
   {
      // We're only interested in trades that are in the trading pool
      // Closed or canceled orders are not interesting for us
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false)
         break;
      
      // Count positions that were opened by our EA
      if((OrderMagicNumber()==MAGICNUMBER))
      {
         // Resize ticket array and save ticket
         ArrayResize(Ticket,count+1);
         Ticket[count] = OrderTicket();

         count++;         
      }
   }   

   // Return open positions
   return(count);
}